=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-17 11:56:54 +0000
+++ bzrlib/branch.py	2005-10-17 21:57:32 +0000
@@ -1258,9 +1258,12 @@
             raise InvalidRevisionNumber(revno)
         
     def sign_revision(self, revision_id, gpg_strategy):
+        plaintext = Testament.from_revision(self, revision_id).as_short_text()
+        self.store_revision_signature(gpg_strategy, plaintext, revision_id)
+
+    def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
         self.lock_write()
         try:
-            plaintext = Testament.from_revision(self, revision_id).as_short_text()
             self.revision_store.add(StringIO(gpg_strategy.sign(plaintext)), 
                                     revision_id, "sig")
         finally:

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-17 11:31:05 +0000
+++ bzrlib/commit.py	2005-10-17 21:57:32 +0000
@@ -79,6 +79,7 @@
                            )
 import bzrlib.gpg as gpg
 from bzrlib.revision import Revision
+from bzrlib.testament import Testament
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
 from bzrlib.inventory import Inventory, ROOT_ID
@@ -321,9 +322,11 @@
         rev_tmp = StringIO()
         serializer_v5.write_revision(self.rev, rev_tmp)
         rev_tmp.seek(0)
+        if self.config.signature_needed():
+            plaintext = Testament(self.rev, self.new_inv).as_short_text()
+            self.branch.store_revision_signature(gpg.GPGStrategy(self.config),
+                                                 plaintext, self.rev_id)
         self.branch.revision_store.add(rev_tmp, self.rev_id)
-        if self.config.signature_needed():
-            self.branch.sign_revision(self.rev_id, gpg.GPGStrategy(self.config))
         mutter('new revision_id is {%s}', self.rev_id)
 
     def _remove_deleted(self):

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2005-10-17 10:50:39 +0000
+++ bzrlib/gpg.py	2005-10-17 21:57:32 +0000
@@ -22,7 +22,18 @@
 
 import bzrlib.errors as errors
 
+class DisabledGPGStrategy(object):
+    """A GPG Strategy that makes everything fail."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        raise errors.SigningFailed('Signing is disabled.')
+
+
 class LoopbackGPGStrategy(object):
+    """A GPG Strategy that acts like 'cat' - data is just passed through."""
 
     def __init__(self, ignored):
         """Real strategies take a configuration."""

=== modified file 'bzrlib/selftest/test_commit.py'
--- bzrlib/selftest/test_commit.py	2005-10-17 11:31:05 +0000
+++ bzrlib/selftest/test_commit.py	2005-10-17 21:57:32 +0000
@@ -21,7 +21,7 @@
 from bzrlib.branch import Branch
 from bzrlib.commit import Commit
 from bzrlib.config import BranchConfig
-from bzrlib.errors import PointlessCommit, BzrError
+from bzrlib.errors import PointlessCommit, BzrError, SigningFailed
 
 
 # TODO: Test commit with some added, and added-but-missing files
@@ -274,3 +274,28 @@
                              branch.revision_store.get('B', 'sig').read())
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_failed_signature(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.revision_store.has_id('A', 'sig'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.DisabledGPGStrategy
+            config = MustSignConfig(branch)
+            self.assertRaises(SigningFailed,
+                              commit.Commit(config=config).commit,
+                              branch, "base",
+                              allow_pointless=True,
+                              rev_id='B')
+            branch = Branch.open('.')
+            self.assertEqual(branch.revision_history(), ['A'])
+            self.failIf(branch.revision_store.has_id('B'))
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-10-17 11:56:54 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-17 21:57:32 +0000
@@ -15,15 +15,17 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+
 from bzrlib.branch import Branch
 from bzrlib.clone import copy_branch
 from bzrlib.commit import commit
 import bzrlib.errors as errors
 from bzrlib.errors import NoSuchRevision, UnlistableBranch, NotBranchError
+import bzrlib.gpg
 from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.trace import mutter
 import bzrlib.transactions as transactions
-from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
 
 # TODO: Make a branch using basis branch, and check that it 
 # doesn't request any files that could have been avoided, by 
@@ -143,7 +145,6 @@
         self.assertEquals(b.pending_merges(), [])
 
     def test_sign_existing_revision(self):
-        import bzrlib.gpg
         branch = Branch.initialize('.')
         branch.commit("base", allow_pointless=True, rev_id='A')
         from bzrlib.testament import Testament
@@ -151,6 +152,12 @@
         self.assertEqual(Testament.from_revision(branch, 'A').as_short_text(),
                          branch.revision_store.get('A', 'sig').read())
 
+    def test_store_signature(self):
+        branch = Branch.initialize('.')
+        branch.store_revision_signature(bzrlib.gpg.LoopbackGPGStrategy(None),
+                                        'FOO', 'A')
+        self.assertEqual('FOO', branch.revision_store.get('A', 'sig').read())
+
 
 class TestRemote(TestCaseWithWebserver):
 

=== modified file 'bzrlib/selftest/testgpg.py'
--- bzrlib/selftest/testgpg.py	2005-10-17 08:15:09 +0000
+++ bzrlib/selftest/testgpg.py	2005-10-17 21:57:32 +0000
@@ -50,3 +50,10 @@
         my_gpg._command_line = lambda:["cat", "-"]
         self.assertEqual("some content\nwith newlines\n",
                          my_gpg.sign("some content\nwith newlines\n"))
+
+
+class TestDisabled(TestCase):
+
+    def test_sign(self):
+        self.assertRaises(errors.SigningFailed,
+                          gpg.DisabledGPGStrategy(None).sign, 'content')

=== modified file 'bzrlib/selftest/testtestament.py'
--- bzrlib/selftest/testtestament.py	2005-10-17 06:57:11 +0000
+++ bzrlib/selftest/testtestament.py	2005-10-17 21:57:32 +0000
@@ -29,7 +29,9 @@
 from bzrlib.trace import mutter
 from bzrlib.osutils import has_symlinks
 
+
 class TestamentTests(TestCaseInTempDir):
+
     def setUp(self):
         super(TestamentTests, self).setUp()
         b = self.b = Branch.initialize('.')
@@ -114,6 +116,14 @@
                       revprops=props)
         t = Testament.from_revision(self.b, 'test@user-3')
         self.assertEqualDiff(t.as_text(), REV_PROPS_TESTAMENT)
+
+    def test___init__(self):
+        revision = self.b.get_revision('test@user-2')
+        inventory = self.b.get_inventory('test@user-2')
+        testament_1 = Testament(revision, inventory).as_short_text()
+        testament_2 = Testament.from_revision(self.b, 
+                                              'test@user-2').as_short_text()
+        self.assertEqual(testament_1, testament_2)
                     
 
 REV_1_TESTAMENT = """\

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2005-10-17 06:57:11 +0000
+++ bzrlib/testament.py	2005-10-17 21:57:32 +0000
@@ -90,19 +90,22 @@
     @classmethod
     def from_revision(cls, branch, revision_id):
         """Produce a new testament from a historical revision"""
-        t = cls()
         rev = branch.get_revision(revision_id)
-        t.revision_id = str(revision_id)
-        t.committer = rev.committer
-        t.timezone = rev.timezone or 0
-        t.timestamp = rev.timestamp
-        t.message = rev.message
-        t.parent_ids = rev.parent_ids[:]
-        t.inventory = branch.get_inventory(revision_id)
-        t.revprops = copy(rev.properties)
-        assert not contains_whitespace(t.revision_id)
-        assert not contains_linebreaks(t.committer)
-        return t
+        inventory = branch.get_inventory(revision_id)
+        return cls(rev, inventory)
+
+    def __init__(self, rev, inventory):
+        """Create a new testament for rev using inventory."""
+        self.revision_id = str(rev.revision_id)
+        self.committer = rev.committer
+        self.timezone = rev.timezone or 0
+        self.timestamp = rev.timestamp
+        self.message = rev.message
+        self.parent_ids = rev.parent_ids[:]
+        self.inventory = inventory
+        self.revprops = copy(rev.properties)
+        assert not contains_whitespace(self.revision_id)
+        assert not contains_linebreaks(self.committer)
 
     def as_text_lines(self):
         """Yield text form as a sequence of lines.
@@ -132,7 +135,7 @@
         r.extend(self._revprops_to_lines())
         if __debug__:
             for l in r:
-                assert isinstance(l, str), \
+                assert isinstance(l, basestring), \
                     '%r of type %s is not a plain string' % (l, type(l))
         return r
 

